/* dstring.c - The dynamic string handling routines used by cpio.
   Copyright (C) 1990, 1991, 1992, 1997 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#include <config.h>

#if STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# if HAVE_STRING_H
#  include <string.h>
# else
#  include <strings.h>
# endif
#endif

#include "dstring.h"

char *xmalloc PARAMS ((size_t n));
char *xrealloc PARAMS ((char *p, size_t n));

/* Initialiaze dynamic string STRING with space for SIZE characters.  */

void
ds_init (string, size)
     dynamic_string *string;
     size_t size;
{
  string->ds_length = size;
  string->ds_string = (char *) xmalloc (size);
  (string->ds_string)[0] = '\0';
}

/* Expand dynamic string STRING, if necessary, to hold SIZE characters.  */

void
ds_resize (string, size)
     dynamic_string *string;
     size_t size;
{
  if (size > string->ds_length)
    {
      string->ds_length = size;
      string->ds_string = (char *)
	xrealloc ((char *) string->ds_string, size);
    }
}

/* Delete dynamic string.  */

void
ds_destroy (string)
     dynamic_string *string;
{
  free (string->ds_string);
  string->ds_string = NULL;
}

/* Dynamic string S gets a string terminated by the EOS character
   (which is removed) from file F.  S will increase
   in size during the function if the string from F is longer than
   the current size of S.
   Return NULL if end of file is detected.  Otherwise,
   Return a pointer to the null-terminated string in S.  */

char *
/* We need a __STDC__ case here because the ANSI prototype in dstring.h
   might have different promotion rules for type "char" in the ANSI vs
   non-ANSI function definition rules.  */
#if __STDC__
ds_fgetstr (FILE *f, dynamic_string *s, char eos)
#else
ds_fgetstr (f, s, eos)
     FILE *f;
     dynamic_string *s;
     char eos;
#endif
{
  int insize;			/* Amount needed for line.  */
  int strsize;			/* Amount allocated for S.  */
  int next_ch;

  /* Initialize.  */
  insize = 0;
  strsize = s->ds_length;

  /* Read the input string.  */
  next_ch = getc (f);
  while (next_ch != eos && next_ch != EOF)
    {
      if (insize >= strsize - 1)
	{
	  ds_resize (s, strsize * 2 + 2);
	  strsize = s->ds_length;
	}
      s->ds_string[insize++] = next_ch;
      next_ch = getc (f);
    }
  s->ds_string[insize++] = '\0';

  if (insize == 1 && next_ch == EOF)
    return NULL;
  else
    return s->ds_string;
}

char *
ds_fgets (f, s)
     FILE *f;
     dynamic_string *s;
{
  return ds_fgetstr (f, s, '\n');
}

char *
ds_fgetname (f, s)
     FILE *f;
     dynamic_string *s;
{
  return ds_fgetstr (f, s, '\0');
}

void
ds_strcat (s, t)
     dynamic_string *s;
     char *t;
{
  ds_resize (s, ds_strlen (s) + strlen (t) + 1);
  strcat (s->ds_string, t);
}

void
ds_strncat (s, t, n)
     dynamic_string *s;
     char *t;
     int n;
{
  size_t length = ds_strlen (s);

  ds_resize (s, length + n + 1);
  strncat (s->ds_string, t, n);
  s->ds_string[length + n] = '\0';
}

char *
ds_get_string (s)
     dynamic_string *s;
{
  return s->ds_string;
}
