/*
** Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef __defined_fmtstring_h
#define __defined_fmtstring_h

/* Include libit dynamic strings */
#include <dstring.h>

/* Data structure for formatted dynamic strings */
struct fmtstring
{
  /* Dynamic string storing the final outcome */
  dynamic_string *string;

  /* Left, right and wrap margins */
  int left, right, wrap;

  /* Current line and column */
  int line, column;

  /* Set to 1 if we wrap once. Set to 0 if we force a line break */
  int wrap_flag;
};

/* Constructor and destructor */
struct fmtstring *new_fmtstring (int left, int right, int wrap);
void delete_fmtstring (struct fmtstring *fs);

/* Change the left, right and wrap margins. Return the old values */
int fmts_set_left (struct fmtstring *fs, int left);
int fmts_set_right (struct fmtstring *fs, int right);
int fmts_set_wrap (struct fmtstring *fs, int wrap);

/* Get current cursor position */
int fmts_get_line (struct fmtstring *fs);
int fmts_get_column (struct fmtstring *fs);

/* Move to column 'column'. If not possible, then go to that
   column on the next line */
void fmts_move_to_column (struct fmtstring *fs, int column);

/* Force a newline */
void fmts_linebreak (struct fmtstring *fs);

/* Catenate text to the string */
void fmts_catenate (struct fmtstring *fs, char *txt);
void fmts_catenate2 (struct fmtstring *fs, char *t1, char *t2);
void fmts_catenate3 (struct fmtstring *fs, char *t1, char *t2, char *t3);
void fmts_catenate4 (struct fmtstring *fs, char *t1, char *t2, 
		     char *t3, char *t4);

/* Return the built string */
char *fmts_string (struct fmtstring *fs);

#endif
