/*
** The minimal hello world program
** Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#include <parg.h>

/*
 * Of course, a radder application will want to use options. To do this,
 * you need and options array, and some hooks. Here is an example.
 */

/* A hook for handling the traditional option. This is the simplest hook
   possible. All we do is set a flat. Note the naming conventions for flags.
   The flag itself must be called NAME_p and the corresponding function must
   be called NAME_hook */

PARG_FLAG_HOOK(traditional)
PARG_FLAG_HOOK(curious)
PARG_GET_OPTIONAL_ARG_HOOK(rude, "Fuck")
PARG_GET_ARG_HOOK(name)

/* Same as in parg-ex5, except that now we have added a "documentation option"
   Documentation options do not affect parsing, but they do work like 
   comments that show up in the --help output. It may be a good idea to
   use at least one documentation option in the beginning of your option list
   to separate your options from the standard options 
   (--help, --usage, --version) */
struct parg_option options[] = {

  { "Modify the greeting format:", 0, 0, 0, 0 },
  { "traditional", 't', 0 , "Use traditional greeting format", 
    traditional_hook },
  { "unexcited", ':', 0, 0, 0},
  { "curious", 'c', 0, "Use an inquisitive greeting format", curious_hook },
  { "puzzled", ':', 0, 0, 0},

  { "Modify the greeting content:", 0, 0, 0, 0},
  { "rude", 'r', "[SWEAR]", "Be rude using swearword SWEAR.", rude_hook },
  { "surly",':', 0, 0, 0},
  { "name", 'n', "NAME", "Greet NAME instead of the world", name_hook },
  { 0, 0, 0, 0, 0}
};

void hello (struct parg *parser);

int 
main (int argc, char *argv[])
{
  struct parg *parser;
  parg_program_bug_address = "<lf@amath.washington.edu>";
  parg_program_documentation = "Hello -- A fully GNITS compliant "
    "hello world program.";
    
  parg_set_invocation (argv[0], PACKAGE, VERSION);
  parser = parg_parse (argc, argv, options);
  hello (parser);
  exit (0);
}

void
hello (struct parg *parser)
{
  /* First thing to print is the greeting */
  if (rude_p)
    printf ("%s you ", rude_argument);
  else
    printf ("Hello ");

  /* Then print the name of the entity that we are greeting */
  if (name_p)
    printf ("%s", name_argument);
  else
    printf ("world");

  /* If we are not being traditional and not inquisitive, then print
     an exclamation mark */
  if ((!traditional_p) && (!curious_p))
    {
      printf ("!");
    }
  /* If we are being inquisitive, then do the question thing instead */
  else if (curious_p)
    {
      printf ("? Are you there?");
    }
  /* Newline */
  printf ("\n");

}

