/*
** The minimal hello world program
** Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#include <parg.h>

/*
 * Of course, a radder application will want to use options. To do this,
 * you need and options array, and some hooks. Here is an example.
 */

/* A hook for handling the traditional option. This is the simplest hook
   possible. All we do is set a flat. Note the naming conventions for flags.
   The flag itself must be called NAME_p and the corresponding function must
   be called NAME_hook */

int traditional_p = 0;
int traditional_hook (struct parg *parser, char *arg)
{
  traditional_p = 1;
}

/* This is the option table. There must be a { 0, 0, 0, 0, 0} entry for
   termination. The first argument is the long name of the option. The
   second argument is the short name of the option. The third argument
   indicates that the option does not take any arguments. The fourth
   argument is the documentation string for the option */
struct parg_option options[] = {
  { "traditional", 't', 0 , "Use traditional greeting format", 
    traditional_hook },
  { 0, 0, 0, 0, 0}
};

int main (int argc, char *argv[])
{
  parg_program_bug_address = "<lf@amath.washington.edu>";
  parg_program_documentation = "Parg example 4 -- A fully GNITS compliant "
    "application.\v"
    "This example includes the use of an option that does not take any "
    "arguments. This option triggers traditional behaviour to the hello "
    "world program.";
    
  parg_set_invocation (argv[0], PACKAGE, VERSION);
  parg_parse (argc, argv, options);

  if (traditional_p)
    printf ("Hello world\n");
  else
    printf ("Hello world!\n");
}
