/*
** The minimal hello world program
** Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#include <parg.h>

/*
 * This application shows how to abbreviate hook generation and how to
 * make aliases for your options
 */

/* Create a hook named traditional_hook that sets the variable traditional_p
   true if the corresponding option appears. This definition must appear 
   before defining the options table */
PARG_FLAG_HOOK(traditional)

/* This table illustrates how to do aliases. Just set the last three arguments
   to 0 and then the option will be an alias to the previous option.
   The "unexcited" option has no short counterpart, so use ':' to indicate 
   that. The 'T' option has no long counterpart, so use NULL to indicate 
   that. In general this is the convention for indicating long-only or
   short-only options. */
struct parg_option options[] = {
  { "traditional", 't', 0 , "Use traditional greeting format", 
    traditional_hook },
  { "unexcited", ':', 0, 0, 0},
  { NULL, 'T', 0, 0, 0},
  { 0, 0, 0, 0, 0}
};

int main (int argc, char *argv[])
{
  parg_program_bug_address = "<lf@amath.washington.edu>";
  parg_program_documentation = "Parg example 5 -- A fully GNITS compliant "
    "application.\v"
    "This example includes the use of an option that does not take any "
    "arguments. This option triggers traditional behaviour to the hello "
    "world program. We also demonstrate how aliases to that option are "
    "handled. ";
    
  parg_set_invocation (argv[0], PACKAGE, VERSION);
  parg_parse (argc, argv, options);

  if (traditional_p)
    printf ("Hello world\n");
  else
    printf ("Hello world!\n");
}
