/*
** The minimal hello world program
** Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#include <parg.h>

/*
 * Of course, a radder application will want to use options. To do this,
 * you need and options array, and some hooks. Here is an example.
 */

/* A hook for handling the traditional option. This is the simplest hook
   possible. All we do is set a flat. Note the naming conventions for flags.
   The flag itself must be called NAME_p and the corresponding function must
   be called NAME_hook */

PARG_FLAG_HOOK(traditional)

/* Same as in parg-ex5, except that now we have added a "documentation option"
   Documentation options do not affect parsing, but they do work like 
   comments that show up in the --help output. It may be a good idea to
   use at least one documentation option in the beginning of your option list
   to separate your options from the standard options 
   (--help, --usage, --version) */
struct parg_option options[] = {
  { "Options that modify the program's behaviour:", 0, 0, 0, 0 },
  { "traditional", 't', 0 , "Use traditional greeting format", 
    traditional_hook },
  { "unexcited", ':', 0, 0, 0},
  { NULL, 'T', 0, 0, 0},
  { 0, 0, 0, 0, 0}
};

int main (int argc, char *argv[])
{
  parg_program_bug_address = "<lf@amath.washington.edu>";
  parg_program_documentation = "Parg example 6 -- A fully GNITS compliant "
    "application.\v"
    "This example includes the use of an option that does not take any "
    "arguments. This option triggers traditional behaviour to the hello "
    "world program. We also demonstrate how aliases to that option are "
    "handled. ";
    
  parg_set_invocation (argv[0], PACKAGE, VERSION);
  parg_parse (argc, argv, options);

  if (traditional_p)
    printf ("Hello world\n");
  else
    printf ("Hello world!\n");
}
